--- Adds extra traits for the players to choose from
-- @author QueuedResonance 2022

require('NPCs/MainCreationMethods')

local TraitFactory = TraitFactory
local Perks = Perks

local getText = getText
local string = string

--- Table of all the traits relevant to ZomboLewd
local TRAITS_LIST = {
	{
		IdentifierType = "Sexaddict",
		Cost = -4,
		Profession = false,
		MutualExclusives = {},
	},
	{
		--Will return in future update.
		IdentifierType = "Masochist",
		Cost = 0,
		Profession = false,
		MutualExclusives = {},
		Callback = function(trait)
			trait:addXPBoost(Perks.Lightfoot, 1)
		end
	},
	{
		--Will return in future update.
		IdentifierType = "Rapist",
		Cost = 0,
		Profession = false,
		MutualExclusives = {},
		Callback = function(trait)
			trait:addXPBoost(Perks.Sneak, 1)
		end
	},
	{
		IdentifierType = "Wimp", --- Fear of rape
		Cost = -4,
		Profession = false,
		MutualExclusives = {},
		-- Trait implemented
	},
	{
		IdentifierType = "Necrophiliac",
		Cost = 3,
		Profession = false,
		MutualExclusives = {},
	},
	{
		--Will return in future update.
		IdentifierType = "Misogynist",
		Cost = 0,
		Profession = false,
		MutualExclusives = {},
	},
	{
		--Will return in future update.
		IdentifierType = "Misandrist",
		Cost = 0,
		Profession = false,
		MutualExclusives = {"Misogynist"},
	},
	{
		IdentifierType = "Everydrop",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "Virgin",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "Inexperienced",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "Experienced",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "Nymphomaniac",
		Cost = 6,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "Innocent",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "Lewd",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "Traumatized",
		Cost = 0,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "drinkerNoob",
		Cost = 1,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "drinkerLight",
		Cost = 1,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "drinkerHeavy",
		Cost = 1,
		Profession = false,
		MutualExclusives = {},
	},
		{
		IdentifierType = "pacify",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "succubus",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		--Implemented but unused, joke trait.
		IdentifierType = "gonorrhea",
		Cost = 0,
		Profession = false,
		MutualExclusives = {},
	},
	{
		--Future update
		IdentifierType = "AIDS",
		Cost = 0,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "Fugly",
		Cost = -3,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "submissive",
		Cost = 3,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "dominate",
		Cost = 3,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "youngBuck",
		Cost = 4,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "AARP",
		Cost = -4,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "inPrime",
		Cost = 2,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "lowEffort",
		Cost = 6,
		Profession = false,
		MutualExclusives = {},
	},
	{
		IdentifierType = "smokesAfter",
		Cost = -2,
		Profession = false,
		MutualExclusives = {},
	},
}

--- Load ZomboLewd specific traits
local function _initTraits()
	for i = 1, #TRAITS_LIST do
		local data = TRAITS_LIST[i]

		local name = getText(string.format("UI_ZL_Trait_%s", data.IdentifierType))
		local desc = getText(string.format("UI_ZL_Trait_%s_Description", data.IdentifierType))
		local newTrait = TraitFactory.addTrait(data.IdentifierType, name, data.Cost, desc, data.Profession)

		--- Activate optional callbacks
		if data.Callback then
			data.Callback(newTrait)
		end

		--- Make it so traits with mutual exclusivity can't select each other
		for v = 1, #data.MutualExclusives do
			TraitFactory.setMutualExclusive(data.IdentifierType, data.MutualExclusives[v])
		end
	end
	--- Manual trait exclusives
	TraitFactory.setMutualExclusive("Virgin", "Inexperienced");
	TraitFactory.setMutualExclusive("Virgin", "Experienced");
	TraitFactory.setMutualExclusive("Virgin", "Nymphomaniac");
	--TraitFactory.setMutualExclusive("Virgin", "Lewd"); Deleted because people can be virgins and lewd.
	TraitFactory.setMutualExclusive("Virgin", "Everydrop"); --I'm aware, yes, but no.
	TraitFactory.setMutualExclusive("Inexperienced", "Experienced");
	TraitFactory.setMutualExclusive("Inexperienced", "Nymphomaniac");
	TraitFactory.setMutualExclusive("Experienced", "Nymphomaniac");
	TraitFactory.setMutualExclusive("Innocent", "Everydrop");
	TraitFactory.setMutualExclusive("Innocent", "Lewd");
	TraitFactory.setMutualExclusive("Innocent", "Nymphomaniac");
	TraitFactory.setMutualExclusive("Innocent", "Sexaddict");
	TraitFactory.setMutualExclusive("Innocent", "succubus");
	TraitFactory.setMutualExclusive("Wimp", "Rapist");
	TraitFactory.setMutualExclusive("Wimp", "Masochist");
	TraitFactory.setMutualExclusive("Wimp", "Nymphomaniac");
	TraitFactory.setMutualExclusive("Wimp", "Lewd");
	TraitFactory.setMutualExclusive("drinkerNoob", "drinkerLight");
	TraitFactory.setMutualExclusive("drinkerNoob", "drinkerHeavy");
	TraitFactory.setMutualExclusive("drinkerLight", "drinkerHeavy");
	TraitFactory.setMutualExclusive("pacify", "succubus");
	TraitFactory.setMutualExclusive("succubus", "Everydrop");
	TraitFactory.setMutualExclusive("dominate", "submissive");
	TraitFactory.setMutualExclusive("dominate", "Wimp");
	TraitFactory.setMutualExclusive("youngBuck", "AARP");
	TraitFactory.setMutualExclusive("youngBuck", "inPrime");
	TraitFactory.setMutualExclusive("AARP", "inPrime");
	TraitFactory.setMutualExclusive("AARP", "lowEffort");
	TraitFactory.setMutualExclusive("lowEffort", "youngBuck");
	TraitFactory.setMutualExclusive("lowEffort", "inPrime");
end

--- Hook up event listeners
Events.OnGameBoot.Add(_initTraits)